library(readr)
library(rvest)
library(stringr)
library(httr)

## script starts; depends on functions below
states <- read_csv('states.csv')
base_url <- "http://apps3.eere.energy.gov/"
buying_url <- str_c(base_url, "greenpower/buying/buying_power.shtml")
dir.create("data")

sapply(states$state, function(state) {                        # for each state
    ## unlink("data", recursive=TRUE)
    try({
    print(state)
    dir.create(str_c("data/", state))
    state_url <- str_c(buying_url, "?state=", state) # build state url
    html <- tryCatch({read_html(state_url)}, error=function(e) {NA})       # read html landing page for state
    write_xml(html, file=str_c("data/", state, ".html")) # write landing page to file

    download_table_urls(html, '//table/tr/td[@headers="utility"]/a', base_url)
    download_table_urls(html, '//table/tr/td[@headers="marketer"]/a', base_url)
    })
})

## script ends

## utility functions start
download_table_urls <-  function(html, xpath, base_url) {

    table_urls <- html %>%
        html_nodes(xpath=xpath)

    for (table_url in table_urls) {     # for each url within first table
        url <- table_url %>%
            html_attr("href") %>%
            str_c(base_url, .)

        r <- GET(url)
        if (status_code(r) == 404) {
            next
        }

        url %>%
            read_html(verbose = TRUE) %>%
            write_html(state, str_trunc(table_url %>% html_text(), 20, ellipsis=""))
    }
}

write_html <- function(htmlbody, state, descriptor) {
    ## input: htmlbody, state name, and descriptor (to describe current file)
    ## output: none; side effect download html into state's folder and name it (hopefully) well
    ## Space at end of descriptor messes things
    descriptor <- gsub(" /?$|[/]", "", descriptor)
    htmlbody %>%
        write_xml(file=str_c("data/", state, "/", descriptor, ".html"))
}
## utility functions end
